#!/usr/bin/perl

my ($topd,$intree,$inrank,$level) = @ARGV;

&credits();
print "#PATH topd*.pl: $topd\n#TREE INFILE: $intree\n#SPECIES INFILE: $inrank\n";

if ( ($topd eq "") && ($intree eq "") && ($inrank eq "") ) {
	exit;
}

###########################
# Read list
###########################
my @list = &get_list($inrank); 

print "\n##########################\n\n";

print "#SPECIES\tSplit Distance\tSIGN\tPERCENTAGE\n";

###########################
# Split distance 
###########################
my $SD = &split_distance($topd,$intree);
print "ORIGINAL\t$SD\t=\t0%\n";

###########################
# rank
###########################
my $PERC = 0;
my $PERC_N = 0;
my %fet = ();
foreach my $l(@list) {
	if ($l ne "") {
		($perc,$n) = &rank($topd,$intree,$SD,$l);
		$PERC += $perc;
		$PERC_N += $n;	
	}
}
print "\n##########################\n\n";
$PERC /= $PERC_N;
printf ("AVERAGE GAIN: %6.3f%\n\n\n", $PERC);


###################################################################################
sub rank {
	my ($topd,$intree,$SD,@L)= @_;

	open (OUT, ">tmp.txt");
	foreach $l(@L) {
		chomp $l;
		$l =~ s/\W$//;
		if ($l ne "") {
			print OUT "$l\n";
		}
	}
	close OUT;

	#prune_tree
	`perl $topd -prune $intree tmp.txt`;

	
	my $perc = 0;
	my $n = 0;

	#split_distance
	my $sd = &split_distance($topd,"pruned_trees");
	my $dist = $sd - $SD;
	
	if ($dist == 0) { 
		$dist = "=";
	}elsif ($dist =~ /^\-/) {
		$dist = "-";
		$perc = (1-($sd/$SD)) * 100;
		print "@L\t";
		printf ("%6.5f\t",$sd);
		print "$dist\t";
		printf ("%6.3f%\n",$perc);
		$n++;
	}else {
		$dist = "+";
	}

	`rm tmp.txt`;
	`rm pruned_trees`;

	return ($perc,$n);
}


sub split_distance {
	my ($topd,$intree) = @_;
	my $sd = 0;
	
	`perl $topd -f $intree -m split -r no`;

	open (IN, "<topd_result.txt");
	while (<IN>) {
		if (/Split Distance \[differents\/possibles\]\: (\d+\.\d+)/)  {
			$sd = $1;
		}elsif (/Split Distance \[differents\/possibles\]\: (\d+)/)  {
			$sd = $1;
		}
	}
	close IN;

	`rm topd_result.txt`;
	
	return $sd;
}

sub get_list {
	my $inrank = shift;

	my @list = ();
	open (IN, "<$inrank");
	while (<IN>) {
	        chomp;
       	 	push @list, $_;
	}
	close IN;

	return @list;
}
	

sub credits() {
	print "\n";
	print "###########################################\n";
	print "#                                         #\n";
	print "#      Ranking disagree method            #\n";
	print "#      (August 2012)                      #\n";
	print "#                                         #\n";
	print "#      Author: Pere Puigbo, PhD           #\n";
	print "#      http://genomes.urv.cat/topd        #\n";
	print "#                                         #\n";
	print "###########################################\n\n";
}



